/************************************************************************
 *                                                                      *
 * File:        rad_api.h		                                *
 *                                                                      *
 * Author(s):   T. Frivold, D.Beyer.  Rooftop Communications.           *
 *                                                                      *
 * Description: Generic Radio API header.                               *
 *                                                                      *
 * Implementations must *NOT* make any assumptions about the numeric    *
 * values of any of the enumerations defined below.  That is, all	*
 * code should use the defined symbols only.				*
 *                                                                      *
 * Similarly, implementations must *NOT* make any assumptions about 	*
 * the size or ordering of any member in structures defined below.	*
 * That is, all code should access the members by structure		*
 * dereference only.							*
 *                                                                      *
 ************************************************************************/
#ifndef _RADIO_API_H_
#define _RADIO_API_H_

#include <dev.h>

/****************************************************************
 *
 * General definitions
 *
 ****************************************************************/

#define RADIO_API_VERSION 0x00010301 /* version 1.3.1 		    */

#ifndef RADIO_MAC_BROADCAST_ADR
#define RADIO_MAC_BROADCAST_ADR	0xFFFFFFFF
#endif

/* 
 * Constant used for indicating an operating constant which is not
 * used or is meaningless for a particular radio.
 */
#define RADIO_UNUSED_CONST	0xFFFF0000

/****************************************************************
 *
 * Radio API device primitives (variables, commands, and signals)
 * and corresponding standard data structures.
 *
 ****************************************************************/

/*
 * Radio API return codes
 */
typedef enum
{
    RADIO_RET_START		= DEV_RET_START,
    RadioRetOk			= DevRetOk,
    RadioRetFail		= DevRetFail,
    RadioRetNoInit		= DevRetNoInit,
    RadioRetTimeOut		= DevRetTimeOut,
    RadioRetMemOut		= DevRetMemOut,
    RadioRetHwFail		= DevRetHwFail,
    RadioRetInvVersion		= DevRetInvVersion,
    RadioRetInvInitData		= DevRetInvInitData,
    RadioRetInvCtlBlockPtr	= DevRetInvCtlBlockPtr,
    RadioRetInvState		= DevRetInvState,
    RadioRetInvCmd		= DevRetInvCmd,
    RadioRetInvVar		= DevRetInvVar,
    RadioRetInvSig		= DevRetInvSig,
    RadioRetInvDev		= DevRetInvDev,
    RadioRetInvPtr		= DevRetInvPtr,
    RadioRetInvSize		= DevRetInvSize,
    RadioRetInvQual		= DevRetInvQual,
    RadioRetInvParam		= DevRetInvParam,
    RadioRetPktRcvFail		= DevPktRetPktRcvFail,
    RadioRetPktXmtFail		= DevPktRetPktXmtFail,
    RadioRetPktRcvError		= DevPktRetPktRcvError,
    RadioRetPktXmtError		= DevPktRetPktXmtError,
    RADIO_RET_NEXT,
} RadioRetType;

/*
 * Definitions for enabling/disabling signals
 */
typedef enum
{
    RadioSigDisable		= DevSigDisable,
    RadioSigEnable		= DevSigEnable,
} RadioSigEnableFlag;

/*
 * Radio API Signals
 */

typedef enum
{
    RADIO_SIG_START		= DEV_SIG_START,
    RadioSigAll			= DevSigAll,
    RadioSigError		= DevSigError,
    RadioSigXmtPkt		= DevPktSigXmtPkt,
    RadioSigRcvPkt		= DevPktSigRcvPkt,
    RadioSigCarrierActive,
    RadioSigCarrierInactive,
    RadioSigCaptureActive,
    RadioSigCaptureInactive,
    RadioSigXmtActive,
    RadioSigRcvActive,
    RadioSigXmtInactive,
    RadioSigRcvInactive,
    RadioSigProcResults,
    RADIO_SIG_NEXT,
} RadioSigType;

/*
 * Radio API Variables
 */
typedef enum
{
    RADIO_VAR_START		= DEV_VAR_START,
    RadioVarVersion		= DevVarVersion,
    RadioVarName		= DevVarName,
    RadioVarQPkts		= DevPktVarQPkts,
    RadioVarQBytes		= DevPktVarQBytes,
    RadioVarMaxPkts		= DevPktVarMaxPkts,
    RadioVarMacAdr		= DevPktVarMacAdr,
    RadioVarMaxMacAdrs		= DevPktVarMaxMacAdrs,
    RadioVarBitRate		= DevPktVarBitRate,
    RadioVarMtu			= DevPktVarMtu,
    RadioVarXmtBurstCnt,
    RadioVarXmtPower,
    RadioVarFreq,
    RadioVarCarrierThresh,
    RadioVarRcvSignal,
    RadioVarRcvNoise,
    RadioVarCode,
    RadioVarCodeRate,
    RadioVarCodeOffset,
    RadioVarFecRate,
    RadioVarSleepMode,
    RadioVarLoopbackMode,
    RadioVarOpConsts,
    RADIO_VAR_NEXT,
} RadioVarType;

/*
 * Radio API Commands
 */

typedef enum
{
    RADIO_CMD_START		= DEV_CMD_START,
    RadioCmdReset		= DevCmdReset,
    RadioCmdXmtPkt		= DevPktCmdXmtPkt,
    RadioCmdRcvPkt		= DevPktCmdRcvPkt,
    RadioCmdDropCapture,
    RadioCmdNativeConsole,
    RadioCmdProcExec,
    RADIO_CMD_NEXT,
} RadioCmdType;

/*
 * Radio Packet Info structure for pkt commands
 *
 * Note that DevPktInfo must be the *first* entry of the RadioDevPktInfo
 * structure, so modules have the option of referring to RadioDevPktInfo
 * objects as only DevPktInfo objects (i.e., with no knowledge of
 * RadioDevPktInfo).
 */

typedef struct _radioPktInfo
{
    DevPktInfo          pktInfo;     /* holds buffer ptrs, etc.     */
    uint32              signature;   /* set to  ID_RAD_PKT          */
    uint32		macAdr;      /* src or dst for pkt	    */
    uint32		errStatus;   /* error status for rcv'd pkts */
    uint32		time_s;      /* Timestamp for recording rcv */
    uint32		time_us;     /*   or controlling xmt time   */
    uint32		power;       /* power of rcv'd pkts         */
} RadioDevPktInfo;

/*
 * Structure used to hold the MAC address/mask list, used with the
 * DevPktVarMacAdr variable primitive.
 */
typedef DevMacAdrInfo RadioDevMacAdrInfo;

/*
 * Radio device operational information, used with the
 *  RadioVarOpConsts variable primitive.
 */
typedef struct _radioDevOpInfo
{
    uint32              signature;   		/* set to ID_RAD_OP_INFO*/
    int32		preambleLen;		/* Units: bits		*/
    int32		xmtFrameLen;		/* Units: bits		*/
    int32		xmtBurstMaxLen;		/* Units: bytes		*/
    int32		xmtQMaxLen;		/* Units: packets	*/
    int32		rcvQMaxLen;		/* Units: packets	*/
    int32		freqIndexNum;		/* Units: freq channels	*/
    int32		freqChangeDelay;	/* Units: us 		*/
    int32		xmtRampUpDelay;		/* Units: us		*/
    int32		xmtRampDownDelay;	/* Units: us		*/
    int32		rcvCaptureLostDelay;	/* Units: us		*/
    int32		rcvCarrierOffDelay;	/* Units: us		*/
    int32		ssCodingGain;		/* Units: dB		*/
    int32		rcvCaptureThresh;	/* Units: dBm		*/
    int32		rcvCaptureSNThresh;	/* Units: dB		*/
    int32		rcvMaintainSyncSNThresh;/* Units: dB		*/
    int32		rcvInterferenceThresh;	/* Units: dBm		*/
} RadioDevOpInfo;
    
/****************************************************************
 *
 * Macros for encoding the 32-bit qualifier value in the DevVarFp(),
 * DevCmdFp() and DevSigFp() function calls.
 *
 ****************************************************************/

#define RADIO_QUAL_CH_ENCODE(index)	DEV_QUAL_INDEX1_ENCODE(index)	
#define RADIO_QUAL_CH_DECODE(qual)	DEV_QUAL_INDEX1_DECODE(qual)	

#define	RADIO_QUAL_ISR		  	DEV_QUAL_ISR

#define	RADIO_QUAL_GET		  	DEV_QUAL_OP_GET
#define	RADIO_QUAL_SET		  	DEV_QUAL_OP_SET
#define	RADIO_QUAL_INC            	DEV_QUAL_OP_INC
#define	RADIO_QUAL_CLR            	DEV_QUAL_OP_CLR

#define	RADIO_QUAL_RCV		  	DEV_QUAL_IO_RCV
#define	RADIO_QUAL_XMT		  	DEV_QUAL_IO_XMT

#define RADIO_QUAL_USER_MASK	  	DEV_QUAL_USER_MASK

#endif /* _RADIO_API_H_ */

